@@ -5,12 +5,11 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2004 James Yonan <jim@yonan.net>
+ *  Copyright (C) 2002-2005 OpenVPN Solutions LLC <info@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  it under the terms of the GNU General Public License version 2
+ *  as published by the Free Software Foundation.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,9 +26,19 @@
 #define COMMON_H
 
 /*
- * Statistics counters.
+ * Statistics counters and associated printf formats.
  */
-typedef unsigned long counter_type;
+#ifdef USE_64_BIT_COUNTERS
+  typedef unsigned long long int counter_type;
+# ifdef WIN32
+#  define counter_format  "%I64u"
+# else
+#  define counter_format  "%llu"
+# endif
+#else
+  typedef unsigned int counter_type;
+# define counter_format   "%u"
+#endif
 
 /*
  * Time intervals
@@ -44,7 +53,6 @@ typedef int interval_t;
 /*
  * Printf formats for special types
  */
-#define counter_format          "%lu"
 #define ptr_format              "0x%08lx"
 #define time_format             "%lu"
 #define fragment_header_format  "0x%08x"
@@ -57,4 +65,12 @@ typedef unsigned long ptr_type;
 /* the --client-config-dir default file */
 #define CCD_DEFAULT "DEFAULT"
 
+/*
+ * This parameter controls the TLS channel buffer size.  Among
+ * other things, this buffer must be large enough to contain
+ * the full --push/--pull list.  If you increase it, do so
+ * on both server and client.
+ */
+#define TLS_CHANNEL_BUF_SIZE 1024
+
 #endif
