@@ -158,6 +158,7 @@ void setenv_str_ex (struct env_set *es,
 		    const unsigned int value_exclude,
 		    const char value_replace);
 
+void setenv_counter (struct env_set *es, const char *name, counter_type value);
 void setenv_int (struct env_set *es, const char *name, int value);
 void setenv_str (struct env_set *es, const char *name, const char *value);
 void setenv_del (struct env_set *es, const char *name);
@@ -238,7 +239,7 @@ void get_user_pass (struct user_pass *up,
 		    const char *prefix,
 		    const unsigned int flags);
 
-void purge_user_pass (struct user_pass *up);
+void purge_user_pass (struct user_pass *up, const bool force);
 
 /*
  * Process string received by untrusted peer before
