@@ -525,6 +525,15 @@ socket_do_accept (socket_descriptor_t sd,
       new_sd = accept (sd, (struct sockaddr *) remote, &remote_len);
     }
 
+#if 0 /* For debugging only, test the effect of accept() failures */
+ {
+   static int foo = 0;
+   ++foo;
+   if (foo & 1)
+     new_sd = -1;
+ }
+#endif
+
   if (!socket_defined (new_sd))
     {
       msg (D_LINK_ERRORS | M_ERRNO_SOCK, "TCP: accept(%d) failed", sd);
@@ -1029,9 +1038,16 @@ link_socket_init_phase2 (struct link_socket *sock,
   struct gc_arena gc = gc_new ();
   const char *remote_dynamic = NULL;
   bool remote_changed = false;
+  int sig_save = 0;
 
   ASSERT (sock);
 
+  if (signal_received && *signal_received)
+    {
+      sig_save = *signal_received;
+      *signal_received = 0;
+    }
+
   /* initialize buffers */
   socket_frame_init (frame, sock);
 
@@ -1223,6 +1239,11 @@ link_socket_init_phase2 (struct link_socket *sock,
        print_sockaddr_ex (&sock->info.lsa->actual, addr_defined (&sock->info.lsa->actual), ":", &gc));
 
  done:
+  if (sig_save && signal_received)
+    {
+      if (!*signal_received)
+	*signal_received = sig_save;
+    }
   gc_free (&gc);
 }
 
