@@ -365,6 +365,11 @@ socket_defined (const socket_descriptor_t sd)
   return sd != SOCKET_UNDEFINED;
 }
 
+/*
+ * Should statistics counters be 64 bits?
+ */
+#define USE_64_BIT_COUNTERS
+
 /*
  * Do we have point-to-multipoint capability?
  */
